//
//  NSDate+RelativeDates.h
//  Tip Keeper
//
//  Created by Matthew Brewer on 1/19/11.
//  Copyright 2011 University of Tennessee: College of Business. All rights reserved.
//

#import <Foundation/Foundation.h>

#define MB_MINUTE 60
#define MB_HOUR   (60 * MB_MINUTE)
#define MB_DAY    (24 * MB_HOUR)
#define MB_5_DAYS (5 * MB_DAY)
#define MB_WEEK   (7 * MB_DAY)
#define MB_MONTH  (30.5 * MB_DAY)
#define MB_YEAR   (365 * MB_DAY)

@interface NSDate (RelativeDates)
- (NSDate*)beginningOfDay:(BOOL)UTC;
- (NSDate*)endOfDay:(BOOL)UTC;
- (NSDate*)beginningOfMonth:(BOOL)UTC;
- (NSDate*)endOfMonth:(BOOL)UTC;
- (NSDate*)beginningOfYear:(BOOL)UTC;
- (NSDate*)endOfYear:(BOOL)UTC;
- (NSDate*)beginningOfWeek:(BOOL)UTC;
- (NSDate*)endOfWeek:(BOOL)UTC;
- (NSDate*)oneWeekAgo;
- (NSUInteger)daysInMonth;

/**
 * Returns the current date with the time set to midnight.
 */
+ (NSDate*)dateWithToday;

/**
 * Returns a copy of the date with the time set to midnight on the same day.
 */
- (NSDate*)dateAtMidnight;

/**
 * Formats the date with 'h:mm a' or the localized equivalent.
 */
- (NSString*)formatTime;

/**
 * Formats the date with 'EEEE, LLLL d, YYYY' or the localized equivalent.
 */
- (NSString*)formatDate;

/**
 * Formats the date according to how old it is.
 *
 * For dates less than a day old, the format is 'h:mm a', for less than a week old the
 * format is 'EEEE', and for anything older the format is 'M/d/yy'.
 */
- (NSString*)formatShortTime;

/**
 * Formats the date according to how old it is.
 *
 * For dates less than a day old, the format is 'h:mm a', for less than a week old the
 * format is 'EEE h:mm a', and for anything older the format is 'MMM d h:mm a'.
 */
- (NSString*)formatDateTime;

/**
 * Formats dates within 24 hours like '5 minutes ago', or calls formatDateTime if older.
 */
- (NSString*)formatRelativeTime;

/**
 * Formats dates within 1 week like '5m' or '2d', or calls formatShortTime if older.
 */
- (NSString*)formatShortRelativeTime;

/**
 * Formats the date with 'MMMM d", "Today", or "Yesterday".
 *
 * You must supply date components for today and yesterday because they are relatively expensive
 * to create, so it is best to avoid creating them every time you call this method if you
 * are going to be calling it multiple times in a loop.
 */
- (NSString*)formatDay:(NSDateComponents*)today yesterday:(NSDateComponents*)yesterday;

/**
 * Formats the date with 'MMMM".
 */
- (NSString*)formatMonth;

/**
 * Formats the date with 'yyyy".
 */
- (NSString*)formatYear;

@end
