//
//  NSDate+RelativeDates.m
//  Tip Keeper
//
//  Created by Matthew Brewer on 1/19/11.
//  Copyright 2011 University of Tennessee: College of Business. All rights reserved.
//

#import "NSDate+RelativeDates.h"


@implementation NSDate (RelativeDates)

- (NSDate*)beginningOfDay:(BOOL)UTC {
	NSDateComponents* components = [[NSCalendar currentCalendar] components:NSDayCalendarUnit|NSMonthCalendarUnit|NSYearCalendarUnit fromDate:self];
	if ( UTC ) {
		components.timeZone = [NSTimeZone timeZoneWithName:@"UTC"];
	}
	return [[NSCalendar currentCalendar] dateFromComponents:components];
}

- (NSDate*)endOfDay:(BOOL)UTC {
	NSDateComponents* components = [[NSCalendar currentCalendar] components:NSDayCalendarUnit|NSMonthCalendarUnit|NSYearCalendarUnit fromDate:self];
	if ( UTC ) {
		components.timeZone = [NSTimeZone timeZoneWithName:@"UTC"];
	}
	components.hour = 23;
	components.minute = 59;
	components.second = 59;
	return [[NSCalendar currentCalendar] dateFromComponents:components];
}

- (NSDate*)beginningOfMonth:(BOOL)UTC {
	NSDateComponents* components = [[NSCalendar currentCalendar] components:NSMonthCalendarUnit|NSYearCalendarUnit fromDate:self];
	return [[[NSCalendar currentCalendar] dateFromComponents:components] beginningOfDay:UTC]; 
}

- (NSDate*)endOfMonth:(BOOL)UTC {
	NSDateComponents* components = [[NSCalendar currentCalendar] components:NSMonthCalendarUnit|NSYearCalendarUnit fromDate:self];
	components.day = [self daysInMonth];
	return [[[NSCalendar currentCalendar] dateFromComponents:components] endOfDay:UTC];
}

- (NSDate*)beginningOfYear:(BOOL)UTC {
	NSDateComponents* components = [[NSCalendar currentCalendar] components:NSYearCalendarUnit fromDate:self];
	return [[[NSCalendar currentCalendar] dateFromComponents:components] beginningOfDay:UTC];
}

- (NSDate*)endOfYear:(BOOL)UTC {
	NSDateComponents* components = [[NSCalendar currentCalendar] components:NSYearCalendarUnit fromDate:self];
	NSDate* date = [[NSCalendar currentCalendar] dateFromComponents:components];
	components.year = 1;
	components.day = -1;
	return [[[NSCalendar currentCalendar] dateByAddingComponents:components toDate:date options:0] endOfDay:UTC];
}

- (NSDate*)beginningOfWeek:(BOOL)UTC {
	NSDateComponents* weekdayComponents = [[NSCalendar currentCalendar] components:NSWeekdayCalendarUnit|NSHourCalendarUnit|NSMinuteCalendarUnit|NSSecondCalendarUnit fromDate:self];	
	NSDateComponents* components = [[[NSDateComponents alloc] init] autorelease];
	components.day = -(weekdayComponents.weekday - [[NSCalendar currentCalendar] firstWeekday]);
	return [[[NSCalendar currentCalendar] dateByAddingComponents:components toDate:self options:0] beginningOfDay:UTC];
}

- (NSDate*)endOfWeek:(BOOL)UTC {
	NSDate* beginning = [self beginningOfWeek:UTC];
	NSDateComponents* components = [[[NSDateComponents alloc] init] autorelease];
	components.day = 7;
	components.second = -1;
	if ( UTC ) {
		components.timeZone = [NSTimeZone timeZoneWithName:@"UTC"];
	}
	return [[NSCalendar currentCalendar] dateByAddingComponents:components toDate:beginning options:0];
}

- (NSDate*)oneWeekAgo {
	NSDateComponents* components = [[[NSDateComponents alloc] init] autorelease];
	components.day = -8;
	return [[NSCalendar currentCalendar] dateByAddingComponents:components toDate:self options:0];
}

- (NSUInteger)daysInMonth {
	return [[NSCalendar currentCalendar] rangeOfUnit:NSDayCalendarUnit inUnit:NSMonthCalendarUnit forDate:self].length;
}






///////////////////////////////////////////////////////////////////////////////////////////////////
+ (NSDate*)dateWithToday {
    NSDateFormatter* formatter = [[NSDateFormatter alloc] init];
    formatter.dateFormat = @"yyyy-d-M";
    
    NSString* formattedTime = [formatter stringFromDate:[NSDate date]];
    NSDate* date = [formatter dateFromString:formattedTime];
    [formatter release];
    formatter = nil;
    
    return date;
}


///////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////
#pragma mark -
#pragma mark Public


///////////////////////////////////////////////////////////////////////////////////////////////////
- (NSDate*)dateAtMidnight {
    NSDateFormatter* formatter = [[NSDateFormatter alloc] init];
    formatter.dateFormat = @"yyyy-d-M";
    
    NSString* formattedTime = [formatter stringFromDate:self];
    NSDate* date = [formatter dateFromString:formattedTime];
    [formatter release];
    formatter = nil;
    
    return date;
}


///////////////////////////////////////////////////////////////////////////////////////////////////
- (NSString*)formatTime {
    static NSDateFormatter* formatter = nil;
    if (nil == formatter) {
        formatter = [[NSDateFormatter alloc] init];
        formatter.dateFormat = NSLocalizedString(@"h:mm a", @"Date format: 1:05 pm");
    }
    return [formatter stringFromDate:self];
}


///////////////////////////////////////////////////////////////////////////////////////////////////
- (NSString*)formatDate {
    static NSDateFormatter* formatter = nil;
    if (nil == formatter) {
        formatter = [[NSDateFormatter alloc] init];
        formatter.dateFormat =
        NSLocalizedString(@"EEEE, LLLL d, YYYY", @"Date format: Monday, July 27, 2009");
    }
    return [formatter stringFromDate:self];
}


///////////////////////////////////////////////////////////////////////////////////////////////////
- (NSString*)formatShortTime {
    NSTimeInterval diff = abs([self timeIntervalSinceNow]);
    
    if (diff < MB_DAY) {
        return [self formatTime];
        
    } else if (diff < MB_5_DAYS) {
        static NSDateFormatter* formatter = nil;
        if (nil == formatter) {
            formatter = [[NSDateFormatter alloc] init];
            formatter.dateFormat = NSLocalizedString(@"EEEE", @"Date format: Monday");
        }
        return [formatter stringFromDate:self];
        
    } else {
        static NSDateFormatter* formatter = nil;
        if (nil == formatter) {
            formatter = [[NSDateFormatter alloc] init];
            formatter.dateFormat = NSLocalizedString(@"M/d/yy", @"Date format: 7/27/09");
        }
        return [formatter stringFromDate:self];
    }
}


///////////////////////////////////////////////////////////////////////////////////////////////////
- (NSString*)formatDateTime {
    NSTimeInterval diff = abs([self timeIntervalSinceNow]);
    if (diff < MB_DAY) {
        return [self formatTime];
        
    } else if (diff < MB_5_DAYS) {
        static NSDateFormatter* formatter = nil;
        if (nil == formatter) {
            formatter = [[NSDateFormatter alloc] init];
            formatter.dateFormat = NSLocalizedString(@"EEE h:mm a", @"Date format: Mon 1:05 pm");
        }
        return [formatter stringFromDate:self];
        
    } else {
        static NSDateFormatter* formatter = nil;
        if (nil == formatter) {
            formatter = [[NSDateFormatter alloc] init];
            formatter.dateFormat = NSLocalizedString(@"MMM d h:mm a", @"Date format: Jul 27 1:05 pm");
        }
        return [formatter stringFromDate:self];
    }
}


///////////////////////////////////////////////////////////////////////////////////////////////////
- (NSString*)formatRelativeTime {
    NSTimeInterval elapsed = abs([self timeIntervalSinceNow]);
    if (elapsed <= 1) {
        return NSLocalizedString(@"just a moment ago", @"");
        
    } else if (elapsed < MB_MINUTE) {
        int seconds = (int)(elapsed);
        return [NSString stringWithFormat:NSLocalizedString(@"%d seconds ago", @""), seconds];
        
    } else if (elapsed < 2*MB_MINUTE) {
        return NSLocalizedString(@"about a minute ago", @"");
        
    } else if (elapsed < MB_HOUR) {
        int mins = (int)(elapsed/MB_MINUTE);
        return [NSString stringWithFormat:NSLocalizedString(@"%d minutes ago", @""), mins];
        
    } else if (elapsed < MB_HOUR*1.5) {
        return NSLocalizedString(@"about an hour ago", @"");
        
    } else if (elapsed < MB_DAY) {
        int hours = (int)((elapsed+MB_HOUR/2)/MB_HOUR);
        return [NSString stringWithFormat:NSLocalizedString(@"%d hours ago", @""), hours];
        
    } else {
        return [self formatDateTime];
    }
}


///////////////////////////////////////////////////////////////////////////////////////////////////
- (NSString*)formatShortRelativeTime {
    NSTimeInterval elapsed = abs([self timeIntervalSinceNow]);
    
    if (elapsed < MB_MINUTE) {
        return NSLocalizedString(@"<1m", @"Date format: less than one minute ago");
        
    } else if (elapsed < MB_HOUR) {
        int mins = (int)(elapsed / MB_MINUTE);
        return [NSString stringWithFormat:NSLocalizedString(@"%dm", @"Date format: 50m"), mins];
        
    } else if (elapsed < MB_DAY) {
        int hours = (int)((elapsed + MB_HOUR / 2) / MB_HOUR);
        return [NSString stringWithFormat:NSLocalizedString(@"%dh", @"Date format: 3h"), hours];
        
    } else if (elapsed < MB_WEEK) {
        int day = (int)((elapsed + MB_DAY / 2) / MB_DAY);
        return [NSString stringWithFormat:NSLocalizedString(@"%dd", @"Date format: 3d"), day];
        
    } else {
        return [self formatShortTime];
    }
}


///////////////////////////////////////////////////////////////////////////////////////////////////
- (NSString*)formatDay:(NSDateComponents*)today yesterday:(NSDateComponents*)yesterday {
    static NSDateFormatter* formatter = nil;
    if (nil == formatter) {
        formatter = [[NSDateFormatter alloc] init];
        formatter.dateFormat = NSLocalizedString(@"MMMM d", @"Date format: July 27");
    }
    
    NSCalendar* cal = [NSCalendar currentCalendar];
    NSDateComponents* day = [cal components:NSDayCalendarUnit|NSMonthCalendarUnit|NSYearCalendarUnit
                                   fromDate:self];
    
    if (day.day == today.day && day.month == today.month && day.year == today.year) {
        return NSLocalizedString(@"Today", @"");
        
    } else if (day.day == yesterday.day && day.month == yesterday.month
               && day.year == yesterday.year) {
        return NSLocalizedString(@"Yesterday", @"");
        
    } else {
        return [formatter stringFromDate:self];
    }
}


///////////////////////////////////////////////////////////////////////////////////////////////////
- (NSString*)formatMonth {
    static NSDateFormatter* formatter = nil;
    if (nil == formatter) {
        formatter = [[NSDateFormatter alloc] init];
        formatter.dateFormat = NSLocalizedString(@"MMMM", @"Date format: July");
    }
    return [formatter stringFromDate:self];
}


///////////////////////////////////////////////////////////////////////////////////////////////////
- (NSString*)formatYear {
    static NSDateFormatter* formatter = nil;
    if (nil == formatter) {
        formatter = [[NSDateFormatter alloc] init];
        formatter.dateFormat = NSLocalizedString(@"yyyy", @"Date format: 2009");
    }
    return [formatter stringFromDate:self];
}


@end
